/// <reference path="./arcgis-js-api.d.ts"/>
define(["require", "exports", "esri/map", "esri/toolbars/draw", "esri/layers/ArcGISDynamicMapServiceLayer", "esri/Color", "esri/symbols/SimpleMarkerSymbol", "esri/symbols/SimpleLineSymbol", "esri/geometry/Extent", "esri/request", "esri/layers/TableDataSource", "esri/layers/LayerDataSource", "esri/layers/FeatureLayer", "esri/tasks/query", "esri/InfoTemplate"], function (require, exports, Map, Draw, ArcGISDynamicMapServiceLayer, Color, SimpleMarkerSymbol, SimpleLineSymbol, Extent, Request, TableDataSource, LayerDataSource, FeatureLayer, Query, InfoTemplate) {
    var SOEClient = (function () {
        function SOEClient() {
            this.PixelTolerance = 10;
            this.MaxQuery = 100;
            // Info specific to the map services being called
            this.ElectricServiceUrl = "http://localhost:6080/arcgis/rest/services/DevSummit/Electric/MapServer";
            this.ExtensionBaseUrl = "http://localhost:6080/arcgis/rest/services/DevSummit/Electric/MapServer/exts/RestSOE1";
            this.DataSource = "Electric_arcgis";
            this.DataPrefix = "Electric.DBO.";
            this.TableNames = {
                "376": "PriOHElectricLineSegment",
                "373": "PriUGElectricLineSegment",
                "364": "Switch",
                "361": "Fuse",
                "362": "DynamicProtectiveDevice"
            };
            this.SubTypes = {
                "Switch": {
                    1: "Overhead Disconnect",
                    2: "Overhead Load Break",
                    3: "Underground Disconnect",
                    4: "Underground Load Break"
                },
                "Fuse": {
                    1: "Current Limiting",
                    2: "Overhead Expulsion",
                    3: "Underground Expulsion"
                },
                "DynamicProtectiveDevice": {
                    1: "Circuit Breaker",
                    6: "Three Phase Electronic Recloser"
                }
            };
        }
        SOEClient.prototype.initClient = function (sMapDiv, sStatusID, sProcessButtonID, sTraceButtonID, sClearButtonID) {
            // Get status element and buttons
            var _this = this;
            var status = document.getElementById(sStatusID);
            var btnProcess = document.getElementById(sProcessButtonID);
            var btnTrace = document.getElementById(sTraceButtonID);
            var btnClear = document.getElementById(sClearButtonID);
            // Set up button events
            btnProcess.onclick = function (evt) { _this.getProcessId(); };
            btnTrace.onclick = function (evt) { _this.activateTool(); };
            btnClear.onclick = function (evt) { _this.clearResults(); };
            // Set up map
            var map = new Map(sMapDiv, {
                showInfoWindowOnClick: true,
                basemap: "streets",
                center: [-88.15, 41.78],
                zoom: 13
            });
            var dynLayer = new ArcGISDynamicMapServiceLayer(this.ElectricServiceUrl);
            dynLayer.id = "Electric_DynamicMap";
            map.addLayer(dynLayer);
            // Finish initializing
            this.startClient(map, status);
        };
        SOEClient.prototype.startClient = function (map, status) {
            var _this = this;
            this._map = map;
            this._status = status;
            // Set up trace symbols
            var colRed = new Color([255, 0, 0]);
            this._symMarkerResult = new SimpleMarkerSymbol(SimpleMarkerSymbol.STYLE_CIRCLE, 12, null, colRed);
            this._symLineResult = new SimpleLineSymbol(SimpleLineSymbol.STYLE_SOLID, colRed, 4);
            this._TraceResultGraphics = [];
            // Set up draw event
            this._draw = new Draw(this._map);
            this._draw.on("draw-end", function (evt) { _this.toolResult(evt); });
        };
        SOEClient.prototype.getProcessId = function () {
            var _this = this;
            var sUrl = this.ExtensionBaseUrl + "/getProcessId";
            Request({
                url: sUrl,
                content: { "f": "json" },
                callbackParamName: "callback",
                load: function (evt) { _this.getProcessIdCallback(evt); }
            });
        };
        SOEClient.prototype.activateTool = function () {
            this.clearResults();
            var sGeom = Draw.POINT;
            this._map.setInfoWindowOnClick(false);
            this._map.disableMapNavigation();
            this._draw.activate(sGeom);
        };
        SOEClient.prototype.clearResults = function () {
            // Remove any trace result graphics
            var iCount = this._TraceResultGraphics.length;
            if (iCount == 0)
                return;
            for (var i = 0; i < iCount; i++)
                this._map.graphics.remove(this._TraceResultGraphics[i]);
            this._TraceResultGraphics = [];
            this.status("OK");
        };
        SOEClient.prototype.status = function (sMsg) {
            this._status.innerHTML = sMsg;
        };
        SOEClient.prototype.getProcessIdCallback = function (evt) {
            if (evt != null) {
                if (evt.pid != null) {
                    this.status("OK");
                    window.alert("PID = " + evt.pid);
                    return;
                }
            }
            this.status("ERROR");
            window.alert("Unexpected result.");
            return;
        };
        SOEClient.prototype.toolResult = function (evt) {
            var _this = this;
            this._draw.deactivate();
            this._map.enableMapNavigation();
            this._map.setInfoWindowOnClick(true);
            var geom = evt.geometry;
            if (geom.type != "point")
                return; // This should never happen
            var pt = geom;
            // Prepare content for request
            var dDist = this._map.extent.getWidth() * this.PixelTolerance / this._map.width;
            var sType = "Edge";
            var myContent = {
                "location": "{x:" + pt.x + ",y:" + pt.y + "}",
                "distance": dDist,
                "featureType": sType,
                "f": "json"
            };
            // Send request
            this.status("Finding network feature...");
            var sUrl = this.ExtensionBaseUrl + "/findNetworkFeature";
            Request({
                url: sUrl,
                content: myContent,
                callbackParamName: "callback",
                load: function (evt) { _this.findNetworkFeatureCallback(evt); }
            });
        };
        SOEClient.prototype.findNetworkFeatureCallback = function (evt) {
            // Check for error
            var _this = this;
            if (evt == null) {
                this.status("ERROR");
                window.alert("Unexpected result.");
                return;
            }
            if (evt.soe_error_content != null) {
                this.status("ERROR");
                window.alert("ERROR: " + evt.soe_error_content);
                return;
            }
            if (evt.soe_message_content != null) {
                this.status("OK");
                window.alert(evt.soe_message_content);
                return;
            }
            // Prepare content for trace request
            var iEID = evt.eid;
            var sType = "Edge";
            var myContent = {
                "flagEID": iEID,
                "flagType": sType,
                "f": "json"
            };
            // Send request
            this.status("Executing trace...");
            var sUrl = this.ExtensionBaseUrl + "/protectiveTrace";
            Request({
                url: sUrl,
                content: myContent,
                callbackParamName: "callback",
                load: function (evt) { _this.protectiveTraceCallback(evt); }
            });
        };
        SOEClient.prototype.protectiveTraceCallback = function (evt) {
            // Check for error
            if (evt == null) {
                this.status("ERROR");
                window.alert("Unexpected result.");
                return;
            }
            if (evt.soe_error_content != null) {
                this.status("ERROR");
                window.alert("ERROR: " + evt.soe_error_content);
                return;
            }
            if (evt.soe_message_content != null) {
                this.status("OK");
                window.alert(evt.soe_message_content);
                return;
            }
            // Gather trace result OIDs
            this._TraceResultClassIDs = [];
            this._TraceResultOIDs = [];
            this._iSwitchBank = 0;
            this._iFuseBank = 0;
            this._iRecloserBank = 0;
            this.status("Retrieving results...");
            for (var i = 0; i < 2; i++) {
                var bEdges = (i == 0);
                var OIDSets = null;
                if (bEdges)
                    OIDSets = evt.traceEdges;
                else
                    OIDSets = evt.stopJunctions;
                for (var ClassID in OIDSets) {
                    var sClassID = ClassID;
                    if (!this.TableNames.hasOwnProperty(sClassID))
                        continue;
                    this._TraceResultClassIDs.push(sClassID);
                    var OIDs = OIDSets[sClassID];
                    this._TraceResultOIDs.push(OIDs);
                    var sTableName = this.TableNames[sClassID];
                    switch (sTableName) {
                        case "Switch":
                            this._iSwitchBank = OIDs.length;
                            break;
                        case "Fuse":
                            this._iFuseBank = OIDs.length;
                            break;
                        case "DynamicProtectiveDevice":
                            this._iRecloserBank = OIDs.length;
                            break;
                    }
                }
            }
            this._ClassIndex = 0;
            this._FeatureIndex = 0;
            this._TraceResultExtent = null;
            this.getResultGraphics(null);
        };
        SOEClient.prototype.getResultGraphics = function (evt) {
            var _this = this;
            if (evt != null) {
                // Loop through returned features and add graphics to results
                var ResultSet = evt.featureSet;
                var ResultGraphics = ResultSet.features;
                var iResultCount = ResultGraphics.length;
                var sResultClassID = this._TraceResultClassIDs[this._ClassIndex];
                var sTitle = this.TableNames[sResultClassID];
                for (var i = 0; i < iResultCount; i++) {
                    var gr = ResultGraphics[i];
                    var geom = gr.geometry;
                    if (geom.type == "point") {
                        gr.setSymbol(this._symMarkerResult);
                        var pt = geom;
                        var env = new Extent(pt.x, pt.y, pt.x, pt.y, pt.spatialReference);
                        if (this._TraceResultExtent == null)
                            this._TraceResultExtent = env;
                        else
                            this._TraceResultExtent = this._TraceResultExtent.union(env);
                    }
                    else
                        gr.setSymbol(this._symLineResult);
                    if (this.SubTypes.hasOwnProperty(sTitle)) {
                        var attr = gr.attributes;
                        var lookup = this.SubTypes[sTitle];
                        var iSubType = attr["SubtypeCD"];
                        var sType = lookup[iSubType];
                        var sID = attr["FacilityID"];
                        var sContent = "Type: " + sType + "<br>Facility ID: " + sID;
                        var info = new InfoTemplate(sTitle, sContent);
                        gr.infoTemplate = info;
                    }
                    this._map.graphics.add(gr);
                    this._TraceResultGraphics.push(gr);
                }
            }
            // Get next batch of object IDs until end is reached for current class ID
            var iClassCount = this._TraceResultClassIDs.length;
            var OIDs = this._TraceResultOIDs[this._ClassIndex];
            var iFeatureCount = OIDs.length;
            if (this._FeatureIndex == iFeatureCount) {
                this._FeatureIndex = 0;
                this._ClassIndex++;
                if (this._ClassIndex == iClassCount) {
                    this.status("OK");
                    if (this._TraceResultExtent != null) {
                        this._TraceResultExtent = this._TraceResultExtent.expand(1.1);
                        this._map.setExtent(this._TraceResultExtent, true);
                    }
                    var sMsg = "Protective Devices Found:";
                    sMsg += "\nSwitch Banks = " + this._iSwitchBank;
                    sMsg += "\nFuse Banks = " + this._iFuseBank;
                    sMsg += "\nRecloser Banks = " + this._iRecloserBank;
                    window.alert(sMsg);
                    return;
                }
                OIDs = this._TraceResultOIDs[this._ClassIndex];
                iFeatureCount = OIDs.length;
            }
            var sClassID = this._TraceResultClassIDs[this._ClassIndex];
            var sName = this.TableNames[sClassID];
            this.status("Retrieving results for " + sName + "..." + this._FeatureIndex);
            var QueryOIDs = [];
            for (var i = 0; i < this.MaxQuery; i++) {
                var iOID = OIDs[this._FeatureIndex];
                QueryOIDs.push(iOID);
                this._FeatureIndex++;
                if (this._FeatureIndex == iFeatureCount)
                    break;
            }
            // Create feature layer
            var srcTable = new TableDataSource();
            srcTable.workspaceId = this.DataSource;
            srcTable.dataSourceName = this.DataPrefix + sName;
            var srcLayer = new LayerDataSource();
            srcLayer.dataSource = srcTable;
            var sUrl = this.ElectricServiceUrl + "/dynamicLayer";
            var options;
            if (this.SubTypes.hasOwnProperty(sName))
                options = { source: srcLayer, outFields: ["*"] };
            else
                options = { source: srcLayer };
            var lyrTrace = new FeatureLayer(sUrl, options);
            lyrTrace.on("query-features-complete", function (evt) { _this.getResultGraphics(evt); });
            // Set up and execute query
            var query = new Query();
            query.objectIds = QueryOIDs;
            query.returnGeometry = true;
            query.outSpatialReference = this._map.spatialReference;
            lyrTrace.queryFeatures(query);
        };
        return SOEClient;
    })();
    return SOEClient;
});
//# sourceMappingURL=SOEClient.js.map