﻿/// <reference path="./arcgis-js-api.d.ts"/>

import Map = require("esri/map");
import Draw = require("esri/toolbars/draw");
import ArcGISDynamicMapServiceLayer = require("esri/layers/ArcGISDynamicMapServiceLayer");
import Color = require("esri/Color");
import Symbol = require("esri/symbols/Symbol");
import SimpleMarkerSymbol = require("esri/symbols/SimpleMarkerSymbol");
import SimpleLineSymbol = require("esri/symbols/SimpleLineSymbol");
import Geometry = require("esri/geometry/Geometry");
import SpatialReference = require("esri/SpatialReference");
import Extent = require("esri/geometry/Extent");
import Point = require("esri/geometry/Point");
import Request = require("esri/request");
import TableDataSource = require("esri/layers/TableDataSource");
import LayerDataSource = require("esri/layers/LayerDataSource");
import FeatureLayer = require("esri/layers/FeatureLayer");
import Query = require("esri/tasks/query");
import FeatureSet = require("esri/tasks/FeatureSet");
import InfoTemplate = require("esri/InfoTemplate");
import Graphic = require("esri/graphic");

export = SOEClient;

class SOEClient
{

	PixelTolerance: number = 10;
	MaxQuery: number = 100;

	// Info specific to the map services being called

	ElectricServiceUrl: string = "http://localhost:6080/arcgis/rest/services/DevSummit/Electric/MapServer";
	ExtensionBaseUrl: string = "http://localhost:6080/arcgis/rest/services/DevSummit/Electric/MapServer/exts/RestSOE1";
	DataSource: string = "Electric_arcgis";
	DataPrefix: string = "Electric.DBO.";
	TableNames: any =
	{
		"376": "PriOHElectricLineSegment",
		"373": "PriUGElectricLineSegment",
		"364": "Switch",
		"361": "Fuse",
		"362": "DynamicProtectiveDevice"
	};
	SubTypes: any =
	{
		"Switch":
		{
			1: "Overhead Disconnect",
			2: "Overhead Load Break",
			3: "Underground Disconnect",
			4: "Underground Load Break"
		},
		"Fuse":
		{
			1: "Current Limiting",
			2: "Overhead Expulsion",
			3: "Underground Expulsion"
		},
		"DynamicProtectiveDevice":
		{
			1: "Circuit Breaker",
			6: "Three Phase Electronic Recloser"
		}
	}

	// Private members

	private _map: Map;
	private _status: HTMLElement;
	private _symMarkerResult: SimpleMarkerSymbol;
	private _symLineResult: SimpleLineSymbol;
	private _draw: Draw;
	private _TraceResultClassIDs: Array<string>;
	private _TraceResultOIDs: Array<Array<number>>;
	private _ClassIndex: number;
	private _FeatureIndex: number;
	private _TraceResultGraphics: Array<Graphic>;
	private _TraceResultExtent: Extent;
	private _iSwitchBank: number;
	private _iFuseBank: number;
	private _iRecloserBank: number;

	initClient(sMapDiv: string, sStatusID: string, sProcessButtonID: string, sTraceButtonID: string, sClearButtonID: string): void
	{

		// Get status element and buttons

		var status: HTMLElement = document.getElementById(sStatusID);
		var btnProcess: HTMLElement = document.getElementById(sProcessButtonID);
		var btnTrace: HTMLElement = document.getElementById(sTraceButtonID);
		var btnClear: HTMLElement = document.getElementById(sClearButtonID);

		// Set up button events

		btnProcess.onclick = (evt) => { this.getProcessId(); };
		btnTrace.onclick = (evt) => { this.activateTool(); };
		btnClear.onclick = (evt) => { this.clearResults(); };

		// Set up map

		var map: Map = new Map(sMapDiv,
			{
				showInfoWindowOnClick: true,
				basemap: "streets",
				center: [-88.15, 41.78],
				zoom: 13
			});
		var dynLayer: ArcGISDynamicMapServiceLayer = new ArcGISDynamicMapServiceLayer(this.ElectricServiceUrl);
		dynLayer.id = "Electric_DynamicMap";
		map.addLayer(dynLayer);

		// Finish initializing

		this.startClient(map, status);

	}

	startClient(map: Map, status: HTMLElement): void
	{

		this._map = map;
		this._status = status;

		// Set up trace symbols

		var colRed: Color = new Color([255, 0, 0]);
		this._symMarkerResult = new SimpleMarkerSymbol(SimpleMarkerSymbol.STYLE_CIRCLE, 12, null, colRed);
		this._symLineResult = new SimpleLineSymbol(SimpleLineSymbol.STYLE_SOLID, colRed, 4);
		this._TraceResultGraphics = [];

		// Set up draw event

		this._draw = new Draw(this._map);
		this._draw.on("draw-end", (evt) => { this.toolResult(evt); });

	}

	getProcessId(): void
	{
		var sUrl: string = this.ExtensionBaseUrl + "/getProcessId";
		Request(
			{
				url: sUrl,
				content: { "f": "json" },
				callbackParamName: "callback",
				load: (evt) => { this.getProcessIdCallback(evt); }
			});
	}

	activateTool(): void
	{
		this.clearResults();
		var sGeom: string = Draw.POINT;
		this._map.setInfoWindowOnClick(false);
		this._map.disableMapNavigation();
		this._draw.activate(sGeom);
	}

	clearResults(): void
	{

		// Remove any trace result graphics

		var iCount: number = this._TraceResultGraphics.length;
		if (iCount == 0)
			return;
		for (var i: number = 0; i < iCount; i++)
			this._map.graphics.remove(this._TraceResultGraphics[i]);
		this._TraceResultGraphics = [];
		this.status("OK");

	}

	private status(sMsg: string): void
	{
		this._status.innerHTML = sMsg;
	}

	private getProcessIdCallback(evt: any): void
	{
		if (evt != null)
		{
			if (evt.pid != null)
			{
				this.status("OK");
				window.alert("PID = " + evt.pid);
				return;
			}
		}
		this.status("ERROR");
		window.alert("Unexpected result.");
		return;
	}

	private toolResult(evt: any): void
	{

		this._draw.deactivate();
		this._map.enableMapNavigation();
		this._map.setInfoWindowOnClick(true);
		var geom: Geometry = evt.geometry;
		if (geom.type != "point")
			return; // This should never happen
		var pt: Point = <Point>geom;

		// Prepare content for request

		var dDist: number = this._map.extent.getWidth() * this.PixelTolerance / this._map.width;
		var sType: string = "Edge";
		var myContent: any =
		{
			"location": "{x:" + pt.x + ",y:" + pt.y + "}",
			"distance": dDist,
			"featureType": sType,
			"f": "json"
		};

		// Send request

		this.status("Finding network feature...");
		var sUrl: string = this.ExtensionBaseUrl + "/findNetworkFeature";
		Request(
			{
				url: sUrl,
				content: myContent,
				callbackParamName: "callback",
				load: (evt) => { this.findNetworkFeatureCallback(evt); }
			});

	}

	private findNetworkFeatureCallback(evt: any): void
	{

		// Check for error

		if (evt == null)
		{
			this.status("ERROR");
			window.alert("Unexpected result.");
			return;
		}
		if (evt.soe_error_content != null)
		{
			this.status("ERROR");
			window.alert("ERROR: " + evt.soe_error_content);
			return;
		}
		if (evt.soe_message_content != null)
		{
			this.status("OK");
			window.alert(evt.soe_message_content);
			return;
		}

		// Prepare content for trace request

		var iEID: number = evt.eid;
		var sType: string = "Edge";
		var myContent: any =
		{
			"flagEID": iEID,
			"flagType": sType,
			"f": "json"
		};

		// Send request

		this.status("Executing trace...");
		var sUrl: string = this.ExtensionBaseUrl + "/protectiveTrace";
		Request(
			{
				url: sUrl,
				content: myContent,
				callbackParamName: "callback",
				load: (evt) => { this.protectiveTraceCallback(evt); }
			});

	}

	private protectiveTraceCallback(evt: any): void
	{

		// Check for error

		if (evt == null)
		{
			this.status("ERROR");
			window.alert("Unexpected result.");
			return;
		}
		if (evt.soe_error_content != null)
		{
			this.status("ERROR");
			window.alert("ERROR: " + evt.soe_error_content);
			return;
		}
		if (evt.soe_message_content != null)
		{
			this.status("OK");
			window.alert(evt.soe_message_content);
			return;
		}

		// Gather trace result OIDs

		this._TraceResultClassIDs = [];
		this._TraceResultOIDs = [];
		this._iSwitchBank = 0;
		this._iFuseBank = 0;
		this._iRecloserBank = 0;
		this.status("Retrieving results...");
		for (var i: number = 0; i < 2; i++)
		{
			var bEdges: boolean = (i == 0);
			var OIDSets: any = null;
			if (bEdges)
				OIDSets = evt.traceEdges;
			else
				OIDSets = evt.stopJunctions;
			for (var ClassID in OIDSets)
			{
				var sClassID: string = <string>ClassID;
				if (!this.TableNames.hasOwnProperty(sClassID))
					continue;
				this._TraceResultClassIDs.push(sClassID);
				var OIDs: Array<number> = OIDSets[sClassID];
				this._TraceResultOIDs.push(OIDs);
				var sTableName: string = this.TableNames[sClassID];
				switch (sTableName)
				{
					case "Switch":
						this._iSwitchBank = OIDs.length;
						break;
					case "Fuse":
						this._iFuseBank = OIDs.length;
						break;
					case "DynamicProtectiveDevice":
						this._iRecloserBank = OIDs.length;
						break;
				}
			}
		}

		this._ClassIndex = 0;
		this._FeatureIndex = 0;
		this._TraceResultExtent = null;
		this.getResultGraphics(null);

	}

	private getResultGraphics(evt: any)
	{

		if (evt != null)
		{

			// Loop through returned features and add graphics to results

			var ResultSet: FeatureSet = evt.featureSet;
			var ResultGraphics: Array<Graphic> = ResultSet.features;
			var iResultCount: number = ResultGraphics.length;
			var sResultClassID: string = this._TraceResultClassIDs[this._ClassIndex];
			var sTitle: string = this.TableNames[sResultClassID];
			for (var i: number = 0; i < iResultCount; i++)
			{
				var gr: Graphic = ResultGraphics[i];
				var geom: Geometry = gr.geometry;
				if (geom.type == "point")
				{
					gr.setSymbol(this._symMarkerResult);
					var pt: Point = <Point>geom;
					var env: Extent = new Extent(pt.x, pt.y, pt.x, pt.y, pt.spatialReference)
					if (this._TraceResultExtent == null)
						this._TraceResultExtent = env;
					else
						this._TraceResultExtent = this._TraceResultExtent.union(env);
				}
				else
					gr.setSymbol(this._symLineResult);
				if (this.SubTypes.hasOwnProperty(sTitle))
				{
					var attr: any = gr.attributes;
					var lookup: any = this.SubTypes[sTitle];
					var iSubType: number = attr["SubtypeCD"];
					var sType: string = lookup[iSubType];
					var sID: string = attr["FacilityID"];
					var sContent: string = "Type: " + sType + "<br>Facility ID: " + sID;
					var info: InfoTemplate = new InfoTemplate(sTitle, sContent);
					gr.infoTemplate = info;
				}
				this._map.graphics.add(gr);
				this._TraceResultGraphics.push(gr);
			}

		}

		// Get next batch of object IDs until end is reached for current class ID

		var iClassCount: number = this._TraceResultClassIDs.length;
		var OIDs: Array<number> = this._TraceResultOIDs[this._ClassIndex];
		var iFeatureCount: number = OIDs.length;
		if (this._FeatureIndex == iFeatureCount)
		{
			this._FeatureIndex = 0;
			this._ClassIndex++;
			if (this._ClassIndex == iClassCount)
			{
				this.status("OK");
				if (this._TraceResultExtent != null)
				{
					this._TraceResultExtent = this._TraceResultExtent.expand(1.1);
					this._map.setExtent(this._TraceResultExtent, true);
				}
				var sMsg: string = "Protective Devices Found:";
				sMsg += "\nSwitch Banks = " + this._iSwitchBank;
				sMsg += "\nFuse Banks = " + this._iFuseBank;
				sMsg += "\nRecloser Banks = " + this._iRecloserBank;
				window.alert(sMsg);
				return;
			}
			OIDs = this._TraceResultOIDs[this._ClassIndex];
			iFeatureCount = OIDs.length;
		}

		var sClassID: string = this._TraceResultClassIDs[this._ClassIndex];
		var sName: string = this.TableNames[sClassID];
		this.status("Retrieving results for " + sName + "..." + this._FeatureIndex);

		var QueryOIDs: Array<number> = [];
		for (var i = 0; i < this.MaxQuery; i++)
		{
			var iOID: number = OIDs[this._FeatureIndex];
			QueryOIDs.push(iOID);
			this._FeatureIndex++;
			if (this._FeatureIndex == iFeatureCount)
				break;
		}

		// Create feature layer

		var srcTable = new TableDataSource();
		srcTable.workspaceId = this.DataSource;
		srcTable.dataSourceName = this.DataPrefix + sName;
		var srcLayer = new LayerDataSource();
		srcLayer.dataSource = srcTable;
		var sUrl: string = this.ElectricServiceUrl + "/dynamicLayer";
		var options: any;
		if (this.SubTypes.hasOwnProperty(sName))
			options = { source: srcLayer, outFields: ["*"] };
		else
			options = { source: srcLayer };
		var lyrTrace: FeatureLayer = new FeatureLayer(sUrl, options);
		lyrTrace.on("query-features-complete", (evt) => { this.getResultGraphics(evt); });

		// Set up and execute query

		var query: Query = new Query();
		query.objectIds = QueryOIDs;
		query.returnGeometry = true;
		query.outSpatialReference = this._map.spatialReference;
		lyrTrace.queryFeatures(query);

	}

} 