﻿/// <reference path="./jimu.d.ts" />
/// <reference path="./arcgis-js-api.d.ts" />
/// <reference path="./dojox.charting.d.ts" />

import BaseWidget = require("jimu/BaseWidget");

import StatisticDefinition = require("esri/tasks/StatisticDefinition");
import Query = require("esri/tasks/query");
import QueryTask = require("esri/tasks/QueryTask");
import FeatureSet = require("esri/tasks/FeatureSet");
import Graphic = require("esri/graphic");

import Chart = require("dojox/charting/Chart");
import PiePlot = require("dojox/charting/plot2d/Pie");
import SimpleTheme = require("dojox/charting/SimpleTheme");
import Highlight = require("dojox/charting/action2d/Highlight");
import Tooltip = require("dojox/charting/action2d/Tooltip");

class WidgetHelper
{

	baseClass: string = "jimu-widget-layerpiechart";
	outStatisticField: string = "MY_STATISTIC";

	createWidgetProps(): any
	{
		var props: any = {};
		props["baseClass"] = this.baseClass;
		props["intWidgetHelper"] = this;  
		// In the startup function context, "this" is the actual widget
		props["startup"] = function () { this.intWidgetHelper.initialize(this); }
		// The remaining functions use lambda expressions so that "this" is the WidgetHelper instance
		//props["onOpen"] = () => { this.onOpen(); }
		//props["onClose"] = () => { this.onClose(); }
		return props;
	}

	private _bw: BaseWidget; // Use this to make BaseWidget calls
	private _divChart: HTMLElement;
	private _sUrl: string;
	private _sStatisticType: string;
	private _sStatisticField: string;
	private _sGroupByField: string;
	private _GroupByLabels: Array<string>;
	private _Values: any;
	private _Colors: any;
	private _Statistics: any;

	initialize(widget: any): void
	{

		// Initialize members

		this._bw = widget;
		this._divChart = widget.divChart;
		this._sUrl = this._bw.config.layerUrl;
		this._sStatisticType = this._bw.config.statisticType;
		this._sStatisticField = this._bw.config.onStatisticField;
		this._sGroupByField = this._bw.config.groupByField;
		this._GroupByLabels = this._bw.config.groupByLabels;
		this._Values = this._bw.config.values;
		this._Colors = this._bw.config.colors;
		this._Statistics = {};

		// Query service for statistics for pie chart
		
		var sd: StatisticDefinition = new StatisticDefinition();
		sd.statisticType = this._sStatisticType;
		sd.onStatisticField = this._sStatisticField;
		sd.outStatisticFieldName = this.outStatisticField;
		var q: Query = new Query();
		q.returnGeometry = false;
		q.outStatistics = [sd];
		q.groupByFieldsForStatistics = [this._sGroupByField];
		var qt: QueryTask = new QueryTask(this._sUrl);
		qt.on("complete", (evt) => { this.getStatistics(evt); });
		qt.on("error", (evt) => { window.alert("ERROR: " + evt.error.message); });
		qt.execute(q);

	}

	private getStatistics(evt: any): void
	{

		// Collate statistics

		var featureSet: FeatureSet = evt.featureSet;
		var Features: Array<Graphic> = featureSet.features;
		var iCount: number = Features.length;
		for (var i: number = 0; i < iCount; i++)
		{
			var gr: Graphic = Features[i];
			var attr: any = gr.attributes;
			var val: any = attr[this._sGroupByField];
			var dVal: number = attr[this.outStatisticField];
			// Find label that goes with value
			var sLabel: string = null;
			for (var j: number = 0; j < this._GroupByLabels.length; j++)
			{
				var sKey: string = this._GroupByLabels[j];
				var v: any = this._Values[sKey];
				if (v == val)
				{
					sLabel = sKey;
					break;
				}
			}
			if (sLabel == null)
				continue;  // Value is not in config
			this._Statistics[sLabel] = dVal;
		}

		// Build chart data and theme colors

		var Data: Array<any> = [];
		var Colors: Array<string> = [];
		for (var i: number = 0; i < this._GroupByLabels.length; i++)
		{
			var sLabel: string = this._GroupByLabels[i];
			var dVal: number = this._Statistics[sLabel];
			var val: any = { y: dVal, stroke: "black", tooltip: sLabel + " = " + dVal };
			Data.push(val);
			Colors.push(this._Colors[sLabel]);
		}

		// Create theme and pie chart

		var myTheme: SimpleTheme = new SimpleTheme({ colors: Colors });
		var pieChart: Chart = new Chart(this._divChart);
		pieChart.setTheme(myTheme);
		pieChart.addPlot("default",
			{
				type: PiePlot,
				radius: 200,
				fontColor: "black",
				labelOffset: -20
			});
		pieChart.addSeries("Status", Data, undefined);
		var anim_h: Highlight = new Highlight(pieChart, "default");
		var anim_t: Tooltip = new Tooltip(pieChart, "default");
		pieChart.render();

	}

}
export = WidgetHelper;

 